// ===================================================================
// $Id: $
//
// glenvmap.h
//
// Class: CEnvironmentMapGL
//
//  The declaration of the class for environment map via OpenGL using shaders
//
// Initial coding by Vlastimil Havran, 2013.

#ifndef __GLENVMAP_H__
#define __GLENVMAP_H__

#include "pgr.h"
#include <GL/freeglut.h>
//#include <GL/glext.h>
#include <GL/gl.h>
#include <GL/glu.h>

// in order to get function prototypes from glext.h, define
//#GL_GLEXT_PROTOTYPES before including glext.h
//#define GL_GLEXT_PROTOTYPES

#include "glext.h"
//#include "glInfo.h"

#include "pgr.h"
#include <cstdio>
#include <cstdlib>
#include <cmath>
#include <vector>
#include <cassert>
#include <cstring>
#include <sys/stat.h>
#include <cmath>
#include <iostream>
#include <iostream>
#include <sstream>
#include <string>
#include <iomanip>

using namespace std;

#include "configg.h"
#include "bconfig.h"
#include "basstr.h"
#include "TexturedMesh.h"

// ----------------------------------------------------
// This class implements textured triangle mesh
class CEnvironmentMapGL
{  
  // internal data connected to vertices
  int cntVertices; // number of vertices to be rendered
  float *vertices; // 3 floats per vertex
  float *texturedata; // the data as texture

  // Arrays for MESH
  // Vertex buffer object
  GLuint vaoHandle;
  // vertex array object for vertices and UV texture coordinates
  GLuint vbovertices; // vbonormals;
  GLint vertexLoc; // attribute index

  // Texture object
  GLuint texobject;
  // texture size
  int tex_width, tex_height;  

  // initialize shaders
  bool createShaderProgramNoNormals();

  // How to render the data
  GLenum renderMode;
  GLuint envShaderProgram; // shader program
  // returns number of vertices to be processed
  int generateEnv(float radius, int cntTriangles);

public:
  // constructor
  CEnvironmentMapGL(): cntVertices(0), vertices(0), texturedata(0),
		       vaoHandle(0), vbovertices(0), vertexLoc(0),
		       texobject(0), tex_width(0), tex_height(0),
		       renderMode(0), envShaderProgram(0) { }
  ~CEnvironmentMapGL() { free(); }
  void init(float radius, int cntTriangles, int renderModeV);
  void free();
  void setRenderMode(int renderModeV);
  // load the files and creates texture with all attributes
  void loadTexture(const char *filename);
  // expects floating point texture in RGB format
  void setTextureData(GLuint envTexID);
  void setTextureData(float *pointer, int tex_width, int tex_height);
  // draw the mesh to the output
  void draw(const glm::mat4 &PVMmatrix, const glm::mat4 &Mmatrix);
};

#endif // __GLENVMAP_H__
