// ===================================================================
// $Id: $
//
// lightenv.h
//
//     The declaration of the main routines for OpenGL viewer application
//
// Initial coding by Jiri Filip and Vlastimil Havran, 2008.
// Rewritten by Vlastimil Havran, 2013 using OpenGL shaders

#ifndef __LIGHTENV_H__
#define __LIGHTENV_H__

#include "configh.h"
#include "brdfmodel.h"
#include "tangles.h"
#include "hdrloader.h"
#include "tgaimg.h"

// Output to files is in OPENEXR format
//#define _ALLOW_OPENEXR_

// render to 32-float buffer without showing to screen ... to be finished 17/9/2013
//#define _RENDER_TOFB

// use 0 .. ratio (evalBRDFcntST/(evalBRDFcntST+evalEMcntST)),
// use 1 .. value 'evalST' clipped to values minValueVIS and maxValueVIS ... for debugging of whatever
extern int visualizationMode;
extern int pseudoColorTechnique; // setting it to 0 switches off the visualization

extern bool readReferenceImage; // if the reference image is read and available
extern CHDRImage *referenceImage; // the reference image

// Restrict rendering to a single pixel or a set of pixels in the center
// Should be commented for standard usage
//#define DEBUG_SINGLE_PIXEL
//#define DEBUG_RESTRICT_RENDERING_TO_CENTER_IMAGE
// Number 3 below means two degrees
const float THRESHOLD_MAX_ANGLE=M_PI/180.f*3.f;

// The real maximum number of samples per pixel taken ... used for sampling EM
// to precompute the ligh directions
const int maxCntSamplesPerPixel = 100000; // used for precomputation of lights from EM maps
//const int maxCntSamplesPerPixel = 10000; // used for precomputation of lights from EM maps

// the default number of lights to be used by the program when started,
// it holds defaultCntSamples <= maxCntSamplesPerPixel
extern int defaultCntSamplesPerPixel;

// to save the values from the image, 0 if nothing is being traced
extern int visModeVariable;

// if to make some visualization how many samples are taken from EM and
// how many from BRDF
extern int pseudoColorTechnique; // setting it to 0 switches off the visualization
// the prefix to save output files
extern char FILEPREFIX[100];

// The rendering algorithm given internally by variable modeCompute
//
// 0 ... single point light
// 1 ... deterministic sampling for all the pixels same EM lights 
// 2 ... random sampling, for all the pixels different sets of directional EM lights
// 3 ... BRDF*cos(theta) sampling only
// 4 ... initial variant of MIS
// 5 ... advanced variant of MIS with storing the samples

// For testing of different methods
const int maxModeCompute = 13;
//const int modeComputeArray[maxModeCompute]={0, 6, -1};
const int modeComputeArray[maxModeCompute]={0, 1, 1, 3, 5, -1};

// ---------------------------------------------------------------------------------------
// Shared variables
//
// The BRDF model used 0...BTF, 1..Phong, 2...BRDFdataFromFile, 3...MatusikBRDFdata
extern int modeBRDF;
// The number of samples to be used to generate image
extern int cntSamplesPerPixel;
// The computing mode used to render the image
extern int modeCompute;
// The alpha blending coefficient for MIS.
// Usage: alpha=0.0 ... use only BRDF, alpha=1.0 - use only EM sampling
extern float alphaMIS;

// The main data structures used during rendering
// The environment map pointer
extern CEnvironmentMap *environMap;
// BRDF model used in the rendering context
extern BRDF *brdf;
// The image used for debugging
extern CEnvironmentMap *debugImage;

// Switch if to randomized precomputed lights, so avoid correlation from EM sampling if set true
extern bool randomizePrecomputedLightsFlag;

// Auxiliary data structures used
extern TAngles *TA;      // object for directions handling and BTF interpolation
extern int actualEnvMap; // specifies the environment map to be used for rendering
extern int useOrigData;  // 0 - using original BTF data, 1- using BTF model
extern int smoothTexturing;
extern int textureSize;
extern float ***textureImage;
extern float *BTFimageF;
extern int win_fbo_width, win_fbo_height;

#define FILEEXT ".hdr"
//#define FILEEXT ".exr"

inline bool
IsEnvMapLighting() {
  return (modeCompute != 0) ? true: false;
}

#endif // __LIGHTENV_H__
