// ================================================================
// $Id: $
//
// render2.cpp
//
// This implements random sampling from environment map, each pixel different set of light sources (random)
//
// Initial coding by Vlastimil Havran, 2013.

#include <cassert>
#include <cmath>

#include "configg.h"
#include "AuxFuncs.h"
//#include "basrnd.h"
#include "ibtfbase.h"
#include "lightenv.h"
#include "brdfmodel.h"
#include "vrmlsup.h"
#include "tangles.h"
#include "render.h"

// ================================================================================
// Random importance sampling at environment map, each pixel has a different (unique)
// set of directional lights
// The samples are drawn online and not precomputed, so it can take substantial time.
// The is the difference to the algorithm in 'render1.cpp'
void         // interpolated vertices, tangents, normals and binormals
computeMode2(float Vint[3], float Tint[3], float Nint[3], float Bint[3], 
	     float t_v, float p_v, int irow, int jcol, int BTFindex, float normF)
{
  float result[3];
  result[0] = result[1] = result[2] = 0.f;
  resetCountersST();
  // counting samples
  int failEM = 0;
  int cntAllSamplesEM = 0;
  int cntValidSamplesEM = 0;
  // for all samples
  const bool allowNewSamples = false;
  int samplesMax = cntSamplesPerPixel;
  for(int l=0; l < samplesMax; l++) {
    // set the values to t_l and p_l
    // Now we compute the directional light from the environment map
    float rnd1 = RandomValue();
    float rnd2 = RandomValue();
    float theta, phi; // direction
    // color from the directional light of an environment map
    float envLightColoursRGB[3]; 
    // use the importance sampling to generate light direction and color
    // and probability of selecting this direction, from two random values
    // It is online and not precomputed, so it can take substantial time.
    // The is the difference to the algorithm in 'render1.cpp'
    float probEM;
    assert(cntAllSamplesEM < maxCntSamplesPerPixel);
    cntAllSamplesEM++;
    environMap[actualEnvMap].importanceSamplingThetaPhi(rnd1, rnd2, theta, phi, 
							envLightColoursRGB, probEM);
    // the conversion to spherical coordinates
    float XYZ[3];
    ConvertThetaPhiToXYZ(theta, phi, XYZ);
    // set values for illumination, view was set in main function
    TA->setIlluDir(XYZ);
    // convert to the right coordinate system on the object' surface
    float t_i, p_i;
    TA->computeIlluAngles(Vint, Tint, Bint, Nint, &t_i, &p_i);
    // t_i and p_i are in radians
    
    // nothing to do with the light since light aims below the normal
    if (t_i >= M_PI/2.0f) {
      failEM++;
      if ((allowNewSamples) && (samplesMax < maxCntSamplesPerPixel)) {
	// take additional samples
	samplesMax++;
      }
      continue;
    }
    // value given by BRDF, includes already cos(theta) factor
    float brdfRGB[3]; 
    cntValidSamplesEM++;
    // the cos(theta) factor of the rendering equation included in the call below
    bool tt = brdf->getBRDFValue(irow, jcol, t_v, p_v, t_i, p_i, brdfRGB);
    for(int isp=0; isp<=2; isp++) {// for all 3 color channels
      // RGB given by BRDF * RGB given by environment map divided by probability from EM
      result[isp] += brdfRGB[isp] * envLightColoursRGB[isp]/probEM;
    }
  }// for all lights
  // statistics update
  failEMcnt += failEM;
  evalEMcnt += cntAllSamplesEM;
  // The number of all samples taken is 'cntAllSamplesEM'
  // The number of samples that do below surface is 'failEM'
  // The number of samples really taken is 'cntValidSamplesEM'
  
  float multIntensity = normF / (float)cntAllSamplesEM;
  for(int isp=0;isp<=2;isp++) {
    BTFimageF[BTFindex+isp] = multIntensity * result[isp];
  }
  setCountersST(0, cntAllSamplesEM);
  return;
}
