// ================================================================
// $Id: $
//
// render3.cpp
//
// This implements random sampling from BRDF*cos(theta)
//
// Initial coding by Vlastimil Havran, 2013.

#include <cassert>
#include <cmath>

#include "configg.h"
#include "AuxFuncs.h"
//#include "basrnd.h"
#include "ibtfbase.h"
#include "lightenv.h"
#include "brdfmodel.h"
#include "vrmlsup.h"
#include "tangles.h"
#include "render.h"

// ================================================================================
// SAMPLING according to BRDF*cos(theta) or another probability density function
void         // interpolated vertices, tangents, normals and binormals
computeMode3(float Vint[3], float Tint[3], float Nint[3], float Bint[3], 
	     float t_v, float p_v, int irow, int jcol,
	     int BTFindex, float normF)
{
  float result[3];
  result[0] = result[1] = result[2] = 0.f;
  resetCountersST();
  float XYZL[3]; // the direction on the surface of an object
  float envLightColoursRGB[3]; //color from the directional light of an environment map
  // use the importance sampling to get the value
  int cntValidSamplesBRDF = 0;
  int failBRDF = 0;
  int cntAllSamplesBRDF=0;
  const bool allowNewSamples = false;

  for(int l=0; l < cntSamplesPerPixel; l++) {
    // now we compute a ray generated by BRDF model according to BRDF*cos(theta)
    // or some other probability density function that is evaluated to 'probBRDF'
    float rnd1 = RandomValue();
    float rnd2 = RandomValue();
    // compute the stochastic sampling for the two random values
    float probBRDF; // probability of sampling at the direction XYZL
    float brdfRGB[3]; // color=energy of environment map at the direction
    assert(cntAllSamplesBRDF < maxCntSamplesPerPixel);
    cntAllSamplesBRDF++;
    if (brdf->spawnStochasticRay(irow, jcol, rnd1, rnd2, // input
				  t_v, p_v, XYZL, brdfRGB, probBRDF)) { // output
      failBRDF++;
      if (allowNewSamples) {
	if (failBRDF > cntSamplesPerPixel) {
	  BTFimageF[BTFindex] = 0;
	  BTFimageF[BTFindex+1] = 0;
	  BTFimageF[BTFindex+2] = 0;
	  return;
	}
	l--;
      }
      continue; // the ray for rnd1,rnd2 does not exist for the sampling
    }    
    // the number of samples taken to compute the integrand
    cntValidSamplesBRDF++;
    // The result ray in XYZL[3] is in the coordinate system (1,0,0),(0,1,0),(0,0,1)
    // at vertex V, given by vector T,B,N. We convert it to the world-space coordinate
    // system by using T,B,N using standard multiplication of vector by matrix
    float XYZ[3];
    // Normal  0, 0, 1 is converted to Nint
    // Tangent 1, 0, 0 is converted to Tint
    // Bint    0, 1, 0 is converted to Bint
    //
    // This is transpose matrix from local coordinate system to world-space system
    // it is possibly only since vectors T,B,N create orthogonal matrix, then inverse
    // matrix is equal to the transposed matrix
    XYZ[0] = Tint[0]*XYZL[0] + Bint[0]*XYZL[1] + Nint[0]*XYZL[2];
    XYZ[1] = Tint[1]*XYZL[0] + Bint[1]*XYZL[1] + Nint[1]*XYZL[2];
    XYZ[2] = Tint[2]*XYZL[0] + Bint[2]*XYZL[1] + Nint[2]*XYZL[2];
    // Set for the conversion to the global coordinate system
    float t_i, p_i;
    ConvertXYZtoThetaPhi(XYZ, t_i, p_i);
    // the light colour from environment map
    float envRGB[3];
    bool readEM = environMap[actualEnvMap].getRGB_ThetaPhi(t_i, p_i, envRGB);
    for(int isp=0; isp<=2; isp++) // for all 3 color channels
      // we multiply BRDF.cos(theta) from BRDF sampling and RGB colour from EM
      // and divide it by probability from BRDF sampling
      result[isp] += brdfRGB[isp] * envRGB[isp] / probBRDF;
  }// for all samples according to BRDF
  // statistics update of global counters
  failBRDFcnt += failBRDF;
  evalBRDFcnt += cntAllSamplesBRDF;

  // The number of all samples taken is 'cntAllSamplesBRDF'
  // The number of samples that do below surface is 'failBRDF'
  // The number of samples really taken is 'cntValidsamplesBRDF'

  float mult;
  if (cntValidSamplesBRDF)
    mult = normF / (float)cntAllSamplesBRDF; // just dividing by number of samples
  else
    mult = 0.f; // black colour

  //float lumBRDF = luminance(result)/(float)cntAllSamplesBRDF;
  //cout << K << " BRDF " << lumBRDF << endl;
  
  for(int isp=0;isp<=2;isp++) {
    BTFimageF[BTFindex+isp] = mult * result[isp];
  }
  setCountersST(cntAllSamplesBRDF, 0);
  return;
}


