#ifndef _PHONG3_H__
#define _PHONG3_H__

#include "configh.h"
#include "brdfmodel.h"

#include <cstring>
#include <string.h>

__BEGIN_SBRDFLIB_HEADER

// -------------------------------------------------------------------
// Implementation using Phong model - white color only
class BRDF_Phong3: public BRDF
{
protected:
  // The specification of the Phong's model that is white color
  float rho_diff[3];
  float rho_spec[3];
  float spec_index;
public:
  BRDF_Phong3(const float rho_diffuse[3], const float rho_specular[3], const float spec_index);
  virtual ~BRDF_Phong3() {}
  virtual BRDF* Dup() const {
    return new BRDF_Phong3(rho_diff, rho_spec, spec_index);
  }

  // returns cosine corrected value of BRDF given the illumination
  // and viewer angle in RGB, returns false on success
  virtual bool getBRDFValue(int irow, int jcol,
			    float theta_view, float phi_view,
			    float theta_il, float phi_il,
			    float RGB[]) const;

  // returns albedo - value between 0.0 and 1.0
  virtual float getAlbedo(int irow, int jcol, // texture position
			  // viewer direction as two angles in radians
			  float theta_view, float phi_view) const;

  // Make stochastic sampling (importance sampling according to BRDF.cos(theta))
  virtual bool spawnStochasticRay(int irow, int jcol, // texture coordinates are ignored
				  float rnd1, float rnd2, // input random variables
				  float thetaV, float phiV, // the direction of a viewer
				  float XYZ[3], // output direction
				  float RGB[], // corresponding color
				  float &prob) const; // and probability

  //get probability of sampling for particular direction of the function BRDF.cos(theta),
  // RGB has to be provided as BRDF.cos(theta)
  virtual float getProbOfSampling(int irow, int jcol, // texture position
				  float theta_view, float phiV, // viewer direction as two angles in radians
				  float theta_i, float phi_i, // illumination direction as two angles in radians
				  const float RGB[]) const; // corresponding color
  virtual int getID() const { return EE_Phong3;}
  const char* getHelp() const;
  virtual void getString(string &outstr) const;

  // manipulation with BRDF model parameters - connection with possible GUI
  virtual int 
  getNumParameters() const;

  virtual const char* 
  getParameterTitle(int parameterIndex) const;

  virtual const char* 
  getParameterDescription(int parameterIndex) const;

  virtual EParameterUIType
  getParameterSyntax(int parameterIndex) const;

  virtual int
  getParameter(int parameterIndex, float values[]) const;

  virtual int
  setParameter(int parameterIndex, float newvalue[]);

  virtual int
  getParameterConstraints(int parameterIndex, float minvals[], float maxvals[]) const;
  
};

__END_SBRDFLIB_HEADER

#endif // _PHONG3_H__
